@@ -9,6 +9,7 @@ from contextlib import contextmanager
 
 from .borders import load_borders_program
 from .boss import Boss
+from .child import set_default_env
 from .cli import create_opts, parse_args
 from .config import cached_values_for, initial_window_size_func
 from .constants import (
@@ -82,10 +83,13 @@ def talk_to_instance(args):
 
 
 def load_all_shaders(semi_transparent=0):
-    load_shader_programs(semi_transparent)
+    load_shader_programs(semi_transparent, load_all_shaders.cursor_text_color)
     load_borders_program()
 
 
+load_all_shaders.cursor_text_color = None
+
+
 def init_glfw(debug_keyboard=False):
     glfw_module = 'cocoa' if is_macos else ('wayland' if is_wayland else 'x11')
     if not glfw_init(glfw_path(glfw_module), debug_keyboard):
@@ -116,8 +120,11 @@ def get_new_os_window_trigger(opts):
 
 def _run_app(opts, args):
     new_os_window_trigger = get_new_os_window_trigger(opts)
-    if is_macos:
+    if False and is_macos:
+        # This is disabled because using custom cursors fails
+        # on dual GPU machines: https://github.com/kovidgoyal/kitty/issues/794
         set_custom_ibeam_cursor()
+    load_all_shaders.cursor_text_color = opts.cursor_text_color
     with cached_values_for(run_app.cached_values_name) as cached_values:
         with startup_notification_handler(extra_callback=run_app.first_window_callback) as pre_show_callback:
             window_id = create_os_window(
@@ -198,6 +205,15 @@ def macos_cmdline():
     return ans
 
 
+def setup_environment(opts, args):
+    extra_env = opts.env.copy()
+    if opts.editor != '.':
+        os.environ['EDITOR'] = opts.editor
+    if args.listen_on:
+        os.environ['KITTY_LISTEN_ON'] = args.listen_on
+    set_default_env(extra_env)
+
+
 def _main():
     try:
         sys.setswitchinterval(1000.0)  # we have only a single python thread
@@ -210,12 +226,12 @@ def _main():
     except Exception:
         if not is_macos:
             raise
-        print('Failed to set locale with LANG:', os.environ.get('LANG'), file=sys.stderr)
-        os.environ.pop('LANG')
+        log_error('Failed to set locale with LANG:', os.environ.get('LANG'))
+        os.environ.pop('LANG', None)
         try:
             locale.setlocale(locale.LC_ALL, '')
         except Exception:
-            print('Failed to set locale with no LANG, ignoring', file=sys.stderr)
+            log_error('Failed to set locale with no LANG, ignoring')
 
     # Ensure kitty is in PATH
     rpath = os.path.dirname(kitty_exe())
@@ -252,8 +268,7 @@ def _main():
             return
     init_glfw(args.debug_keyboard)  # needed for parsing native keysyms
     opts = create_opts(args)
-    if opts.editor != '.':
-        os.environ['EDITOR'] = opts.editor
+    setup_environment(opts, args)
     try:
         with setup_profiling(args):
             # Avoid needing to launch threads to reap zombies
